# coding=utf-8
from __future__ import absolute_import, print_function

import os

from suanpan.proxy import Proxy

TYPE = os.environ.get("SP_SCREENSHOTS_TYPE", "index")
PATTERN = os.environ.get("SP_SCREENSHOTS_PATTERN")
STORAGE_KEY = os.environ.get("SP_SCREENSHOTS_STORAGE_KEY")


class Screenshots(Proxy):
    MAPPING = {
        "index": "suanpan.screenshots.base.ScreenshotsIndexSaver",
        "time": "suanpan.screenshots.base.ScreenshotsTimeSaver",
    }
    DEFAULT_PATTERN_MAPPING = {"index": "{index}.png", "time": "{time}.png"}

    def __init__(self, *args, **kwargs):
        kwargs.setdefault("type", TYPE)
        kwargs.setdefault("name", STORAGE_KEY)
        kwargs.setdefault(
            "pattern", PATTERN or self.DEFAULT_PATTERN_MAPPING.get(kwargs["type"])
        )
        super(Screenshots, self).__init__()
        self.setBackend(*args, **kwargs)


screenshots = Screenshots()



import suanpan
from suanpan.app import app
from suanpan.arguments import String


@app.output(String(key="outputData1"))
def test_app(_):
    print("test_app")
    return "test_app"


@app.beforeInit
def test_app_before_init():
    print("app_before_init")


@app.afterInit
def test_app_after_init(_):
    print("app_after_init")


@app.beforeCall
def test_app_before_call(_):
    print("app_before_call")


@app.afterCall
def test_app_after_call(_):
    print("app_after_call")


@app.beforeSave
def test_app_before_save(_):
    print("app_before_save")


@app.afterSave
def test_app_after_save(_):
    print("app_after_save")


@app.trigger(interval=100)
@app.trigger.output(String(key="outputData1"))
def test_trigger(_):
    print("test_trigger")
    return "test_trigger"


@app.trigger.beforeInit
def test_trigger_before_init():
    print("trigger_before_init")


@app.trigger.afterInit
def test_trigger_after_init(_):
    print("trigger_after_init")


@app.trigger.beforeCall
def test_trigger_before_call(_):
    print("trigger_before_call")


@app.trigger.afterCall
def test_trigger_after_call(_):
    print("trigger_after_call")


@app.trigger.beforeSave
def test_trigger_before_save(_):
    print("trigger_before_save")


@app.trigger.afterSave
def test_trigger_after_save(_):
    print("trigger_after_save")


if __name__ == "__main__":
    suanpan.run(app)



# coding=utf-8
from __future__ import absolute_import, print_function

from suanpan import arguments as base
from suanpan.arguments import auto
from suanpan.dw import arguments as dw
from suanpan.model import arguments as model
from suanpan.storage import arguments as storage

Int = base.Int
Float = base.Float
Bool = base.Bool
List = base.List
ListOfString = base.ListOfString
ListOfInt = base.ListOfInt
ListOfFloat = base.ListOfFloat
ListOfBool = base.ListOfBool
IntOrFloat = base.IntOrFloat
IntFloatOrString = base.IntFloatOrString
BoolOrString = base.BoolOrString
StringOrListOfFloat = base.StringOrListOfFloat

File = storage.File
Folder = storage.Folder
Data = storage.Data
Csv = storage.Csv
Npy = storage.Npy
Visual = storage.Visual
Model = model.CommonModel
H5Model = storage.H5Model
Checkpoint = storage.Checkpoint
JsonModel = storage.JsonModel

Table = dw.Table


class Json(auto.AutoArg):
    MAPPING = {
        "inputs": storage.Json,
        "outputs": storage.Json,
        "params": base.Json,
        "columns": base.Json,
    }


class String(auto.AutoArg):
    MAPPING = {
        "inputs": storage.String,
        "outputs": storage.String,
        "params": base.String,
        "columns": base.String,
    }



# coding=utf-8
from __future__ import absolute_import, print_function

from suanpan.proxy import Proxy


class AutoArg(Proxy):
    TYPE_KEY = "argtype"
    DEFAULT_TYPE = "params"

    def _setBackend(self, BackendClass, *args, **kwargs):
        kwargs.pop(self.TYPE_KEY)
        return super(AutoArg, self)._setBackend(BackendClass, *args, **kwargs)



# coding=utf-8
from __future__ import absolute_import, print_function

from datetime import datetime

from suanpan.objects import Context, HasName


class StateBase(HasName):
    def __init__(self, *args, **kwargs):
        super(StateBase, self).__init__()
        self._args = args
        self._kwargs = kwargs
        self.init(*self._args, **self._kwargs)

    def init(self, *args, **kwargs):
        self.current = Context(*args, **kwargs)

    def updateCurrent(self, *args, **kwargs):
        raise NotImplementedError("Method not implemented!")

    def update(self, *args, **kwargs):
        raise NotImplementedError("Method not implemented!")

    def clean(self, *args, **kwargs):
        raise NotImplementedError("Method not implemented!")

    def reset(self, *args, **kwargs):
        self.clean(*args, **kwargs)
        self.init(*self._args, **self._kwargs)


class Loader(StateBase):
    def load(self, *args, **kwargs):
        self.updateCurrent()
        return self.update(*args, **kwargs)


class IndexLoader(Loader):
    def __init__(self, index=0, **kwargs):
        super(IndexLoader, self).__init__(index=index, **kwargs)

    def updateCurrent(self, index=None):
        self.current.index = index or self.current.index + 1
        return self.current


class Saver(StateBase):
    def save(self, *args, **kwargs):
        self.updateCurrent()
        return self.update(*args, **kwargs)


class IndexSaver(Saver):
    def __init__(self, index=0, **kwargs):
        super(IndexSaver, self).__init__(index=index, **kwargs)

    def updateCurrent(self, index=None):
        self.current.index = index or self.current.index + 1
        return self.current


class TimeSaver(Saver):
    def __init__(self, time=None, **kwargs):
        time = time or datetime.now()
        super(TimeSaver, self).__init__(time=time, **kwargs)

    def updateCurrent(self, time=None):
        self.current.time = time or datetime.now()
        return self.current



# coding=utf-8
from __future__ import absolute_import, print_function

import functools


def instancemethod(func):
    @functools.wraps(func)
    def _dec(self, *args, **kwargs):  # pylint: disable=unused-argument
        return func(*args, **kwargs)

    return _dec


def bindmethod(instance, func, name=None):
    name = name or func.__name__
    func = func.__get__(instance, instance.__class__)
    setattr(instance, name, func)
    return instance


def bind(instance, func, name=None):
    return bindmethod(instance, instancemethod(func), name=name)
